#! /usr/bin/env python
#
#   Copyright (C) 2006, 2007, One Laptop Per Child Copyright (C) 2007
#   Benjamin Mako Hill <mako@atdot.cc>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, see <http://www.gnu.org/licenses/>.

import os
import sys
import codecs
import shutil
from ConfigParser import ConfigParser

# build a library path   ### hardcoded here as per mstone's suggestion
core_library_path = '/usr/share/activities' 
home_library_path = '/home/olpc/Library' # pull from sugar dynamically
output_path = '/home/olpc/.library_pages'

# create/(re)initialize the .library_pages directory 
try:
    if not os.path.exists(output_path):
        os.mkdir(output_path)
    else: # clear out the directory if it does
        for file in os.listdir(output_path):
            if not os.path.isdir(os.path.join(output_path, file)): continue
            shutil.rmtree(os.path.join(output_path, file))
except:
    print >>sys.stderr, \
        "E: could not initialize library directory '%s'" % output_path
    sys.exit()

# load in the templating system and set up the filesystem loader
try:
    import jinja
except ImportError:
    sys.path.append(os.path.join(os.path.dirname(__file__), 'lib'))
    import jinja

common_url = os.path.abspath(os.path.dirname(__file__))
config_path = os.path.join(os.path.dirname(__file__), 'config')
category_list_filename = os.path.join(config_path, 'categories.info')

ignore_dirs = ['config', 'css', 'js', 'lib', '.git']
main_categories = ['activities', 'books', 'health',
                   'images', 'media', 'science', 'search', 'other']
bundle_list = []

# generate category data
def generate_categories():
    category_data = {}
    cp = ConfigParser()
    cp.readfp(codecs.open(category_list_filename, 'r', 'utf-8'))
    for section in cp.sections():
        section_info = {}
        for name, value in cp.items(section):
            section_info[name] = value

        section_info['bundles'] = []
        category_data[section_info['name']] = section_info

    return category_data

category_data = generate_categories()

def add_dir_to_bundle_list(base_path):
    if not os.path.isdir(base_path): 
	return
    else:
    	dir_list = os.listdir(base_path)

    for bundle in dir_list:
        bundle = os.path.join(base_path, bundle).strip()

        # Skip scripts folder
        if os.path.basename(bundle) in ignore_dirs \
            or not os.path.isdir(bundle):
            continue
        elif not os.path.exists(os.path.join(bundle, 'library/library.info')):
            continue

        # check the files to make sure that the templates are encoded in
        # UTF-8 and provide a warning if they are not
        try:
            for index in ['index.tmpl', 'index.html']:
                if os.path.exists(os.path.join(bundle, index)):
                    codecs.open(os.path.join(bundle, index),
                                'r', 'utf-8').read().encode('utf-8')
        except UnicodeDecodeError:
            print >>sys.stderr, \
                "W: index in '%s' is not UTF-8.  Skipping." \
                 % os.path.basename(bundle)
            continue

        # add bundle to bundle_list unless we've seen it already
        if not os.path.basename(bundle) in \
            map(lambda fn: os.path.basename(fn), bundle_list):
            bundle_list.append(bundle)



# build bundle_list
add_dir_to_bundle_list(core_library_path)
add_dir_to_bundle_list(home_library_path)

# iterate through the list of valid bundles and generate the html if its
# templates are present
for bundle in bundle_list:
    info_file_path = os.path.join(bundle + '/library/library.info')

    # Check if there is a info file
    if os.path.isfile(info_file_path):
        new_bundle = {}
        new_bundle['directory'] = '"' + bundle + '"'
        new_bundle['base_url'] = os.path.dirname(bundle)
        
        cp = ConfigParser()
        cp.readfp(codecs.open(info_file_path, 'r', 'utf-8'))
        
        for item in cp.items('Library'):
            new_bundle[item[0]] = item[1]
	bundle_category = new_bundle['category']
        if new_bundle['category'] in main_categories:
	    bundle_category = new_bundle['category']
	else:
	    bundle_category = 'other'
        category_data[bundle_category]['bundles'].append(new_bundle) 

# write out the HTML for the index file
def categories():
    categories = []
    cp = ConfigParser()
    cp.readfp(codecs.open(category_list_filename, 'r', 'utf-8'))
    for section in cp.sections():
        if len(category_data[section]['bundles']) > 0:
            categories.append(category_data[section])
    return categories


# return a category for a given bundle
def get_cat_for_bundle(bundle):
    bundle = bundle.strip()
    for cat in category_data.keys():
        if bundle in map(lambda x: x['name'],
            category_data[cat]['bundles']):
            return cat
    return None


def render(file, vars={}, **kw):
    jinja_env = jinja.Environment(
        loader= jinja.FileSystemLoader(os.path.dirname(__file__)))

    if kw.has_key('is_html') and kw['is_html']:
        tmpl = jinja_env.from_string(build_template(file))
    else:
        tmpl = jinja_env.get_template(file)

    vars['categories'] = categories()
    return tmpl.render(vars) 

# render the website
#render('index')

def build_template(html_file):
    raw_tmpl = '{% extends "layout.tmpl" %}\n{% block content %}\n'
    raw_tmpl += codecs.open(html_file, 'r', 'utf-8').read()
    raw_tmpl += '\n{% endblock %}'
    return raw_tmpl

vars = {'categories' : categories(),
        'common_url' : common_url,
        'output_path' : output_path }

def output_page(bundle, html):
    output_dir = os.path.join(output_path, os.path.basename(bundle))
    os.mkdir(output_dir)
    
    output_file = codecs.open(
        os.path.join(output_dir, 'bundle_index.html'), 'w', 'utf-8')
    print >>output_file, html

# render any bundles that have index.tmpl files that exist
for bundle in bundle_list:
    bundle_category = get_cat_for_bundle(os.path.basename(bundle))

    vars['base_url'] = bundle
    if bundle_category in main_categories:
        vars['category'] = bundle_category
    else:
        vars['category'] = 'other'
    vars['primary_color'] = category_data[vars['category']]['primarycolor']
    vars['category_name'] = category_data[vars['category']]['name']
    vars['bundle'] = os.path.basename(bundle)

    vars['secondary_color'] = category_data[vars['category']]['secondarycolor']

    if os.path.isfile(bundle + '/index.tmpl'):
        html = render(bundle + '/index.tmpl')
    elif os.path.isfile(bundle + '/index.html'):
        html = render(bundle + '/index.html', vars, is_html=True)
    else:
        continue

    output_page(bundle, html)
